/*
  Author : kolja <gava@bergamoblog.it>
  Creation : 29/11/2003
  Licence : GPL

*********************************************************************
 File		: 	$RCSfile: eci-load2_init.c,v $
 Version	:	$Revision: 1.1.2.1 $
 Modified by	:	$Author: kolja_gava $ ($Date: 2004/02/02 13:29:23 $)
*********************************************************************
  Used to Initialize (steps 1 & 2 of 3) the usb device having chipset GS7470. 
*/

#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <sys/time.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/signal.h>
#include <sys/wait.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <errno.h>
#include "pusb.h"
#include "util.h"
#include "semaphore.h"
#include "eci-load2_init.h"
#include "gsinterface.h"

#define TIMEOUT 2000

struct usb_block
{
	unsigned char   request_type;
	unsigned char   request;
	unsigned short  value;
	unsigned short  index;
	unsigned short  size;
	unsigned char* buf; /* buf's content is stored in the .bin file
							only for OUT request (ie request_type & 0x80)=0 */
};

/* InsertChar:
  Inserts a char in to an array of char at a specified
  position.
*/
char* InsertChar(char*src, char chr,int pos){
	char *target;
	target=malloc((strlen(src)+1));
	strncpy(target, src, strlen(src)+1);
	strcpy((target+pos+1),(src+pos));
    target[pos]=chr;
    return(target);
}

int eci_load2_init(char* file, pusb_device_t dev, int shared_sem, int option_verbose){
    char* afile;
    char* bfile;
 	FILE* fp ;
	struct usb_block block;
	long size;
	int n = 0, r, i;
	key_t kEP;
	int sEpId;
	char* sEpResponse;
	char* sEpResponseOK;
     
    afile = InsertChar(file, 'a',strlen(file)-4);
    bfile = InsertChar(file, 'b',strlen(file)-4);
    
	/* open 'a' file -- INIT*/

	fp = fopen(afile, "rb");
	if (fp == NULL){
		if (option_verbose){
			printf("File init Synch missing.. try to go through (file : %s)\n", afile);
		}
		return(1);
	}
	
	fseek(fp, 0, SEEK_END);
	size = ftell(fp);
	fseek(fp, 0, SEEK_SET);
	
	memset(&block, 0, sizeof(block));

	while (ftell(fp) < size)	{
		n++;
		if (!usb_block_read(fp, &block)){
			printf("can't read block\n");
#ifndef TESTECI
			pusb_close(dev);
#endif
			fclose(fp);
			return(0);
		}

		if (!option_verbose)
				printprogres();

		if (option_verbose)
			printf ("Block %3d: request_type=%02x request=%02x value=%04x index=%04x size=%04x \n",
					n, block.request_type, block.request, block.value,block.index, block.size);

#ifndef TESTECI
		if ((r=pusb_control_msg(dev, block.request_type, block.request,
							 block.value, block.index, block.buf,
							 block.size, TIMEOUT)) != block.size){
			if (option_verbose)
				printf("failed r=%d\n", r);

			pusb_close(dev);
			fclose(fp);
			return(0);
		}

		if (r != block.size)
			printf("(expected %d, got %d) ", block.size, r);
#else
		r = block.size;
#endif
		if (option_verbose){
			if ((block.request_type & 0x80))
			{
				for (i = 0; i < r; i++)
					printf("%02x ", block.buf[i]);
				printf("\n");
			}
		}

#ifndef TESTECI
		fflush(stdout);
		fflush(stderr);
#endif

	}
#ifndef TESTECI
#endif

	if (ftell(fp) != size)
		printf("file size = %ld <=> bytes read = %ld\n", size, ftell(fp));
		
	fclose(fp);

	/* open 'b' file -- STEP1*/

	fp = fopen(bfile, "rb");
	if (fp == NULL){
		if (option_verbose){
			printf("File STEP1 missing.. try to go through (file : %s)\n", bfile);
		}
		return(1);
	}
	
	fseek(fp, 0, SEEK_END);
	size = ftell(fp);
	fseek(fp, 0, SEEK_SET);
	
	/* read first to byte containing exit codes - kolja */	
	sEpResponseOK = malloc(2);
	if ((r = fread(sEpResponseOK, sizeof(char), 2, fp)) != sizeof(char)*2)
	{
		printf("read ep Reponse OK : read %d bytes instead of %d\n", r, sizeof(char)*2);
		return(0);
	}
	if (option_verbose)
		printf ("responseOK : %02x%02x\n", sEpResponseOK[0], sEpResponseOK[1]);

    /* make the key: */
    if ((kEP = ftok("/" CONF_DIR "/eciadsl.conf", 'E')) == -1) {
        printf("ERROR - failed to share ep response (ftok)\n");
        return(0);
    }

    /* connect to (and possibly create) the segment: */
    if ((sEpId = shmget(kEP, 2, 0644)) == -1) {
        printf("ERROR - failed to share ep response (shmget)\n");
        return(0);
    }
    
    /* attach to the segment to get a pointer to it: */
    sEpResponse = shmat(sEpId, (void *)0, 0);
    if (sEpResponse == (char *)(-1)) {
        printf("ERROR - failed to share ep response (shmat)\n");
        return(0);
    }
    
	memset(&block, 0, sizeof(block));

	while ((ftell(fp) < size) && (memcmp(sEpResponse, sEpResponseOK,2)==-1))	{
		n++;

/*	   printf ("test : 0 - %02x %02x ; 1 - %02x %02x\n", sEpResponse[0], sEpResponseOK[0], sEpResponse[1], sEpResponseOK[1]); */

		if (!usb_block_read(fp, &block)){
			printf("can't read block\n");
#ifndef TESTECI
			pusb_close(dev);
#endif
			fclose(fp);
			return(0);
		}

		if (!option_verbose)
				printprogres();

		if (option_verbose)
			printf ("Block %3d: request_type=%02x request=%02x value=%04x index=%04x size=%04x\n",
					n, block.request_type, block.request, block.value,block.index, block.size);

#ifndef TESTECI
		if ((r=pusb_control_msg(dev, block.request_type, block.request,
							 block.value, block.index, block.buf,
							 block.size, TIMEOUT)) != block.size){
			if (option_verbose)
				printf("failed r=%d\n", r);

			pusb_close(dev);
			fclose(fp);
			return(0);
		}

		if (r != block.size)
			printf("(expected %d, got %d) ", block.size, r);
#else
		r = block.size;
#endif
		if (option_verbose){
			if ((block.request_type & 0x80))
			{
				for (i = 0; i < r; i++)
					printf("%02x ", block.buf[i]);
				printf("\n");
			}
		}

#ifndef TESTECI
		if (block.request_type == 0x40 && block.request == 0xdc
			&& block.value == 0x00 && block.index == 0x00)
		{
			struct timeval tv1, tv2;

			if (option_verbose)
			{
				printf("waiting.. ");
				
				fflush(stdout);
				fflush(stderr);

				gettimeofday(&tv1, NULL);
			}

            /* decrement the shared semaphore, thus waiting for the child */
            if (semaphore_decr(shared_sem, 1) == -1)
            {
                printf("ECI load 2 init: error decrementing the shared semaphore");
            }
			if (option_verbose)
			{
				gettimeofday(&tv2, NULL);
				printf("waited for %ld ms\n",
					   (long)(tv2.tv_sec - tv1.tv_sec) * 1000
					   + (long)(tv2.tv_usec - tv1.tv_usec) / 1000);
			}
		}

		fflush(stdout);
		fflush(stderr);
#endif
	/*restart reading STEP1 file ('b' file) if no valid EP response received and file is at the and  - kolja*/
		if(ftell(fp)==size){
			fseek(fp, 2, SEEK_SET);
		}	
	}
#ifndef TESTECI
#endif

    if (shmdt(sEpResponse) == -1) {
        printf("ERROR - failed to release shared ep response (shmat)\n");
    }

	fclose(fp);
	return(1);   
}
