/*
  Author : kolja <gava@bergamoblog.it>
  Creation : 29/11/2003
  Licence : GPL

*********************************************************************
 File		: 	$RCSfile: gs7470.c,v $
 Version	:	$Revision: 1.1.2.2 $
 Modified by	:	$Author: kolja_gava $ ($Date: 2004/08/01 15:36:52 $)
*********************************************************************
  GlobeSpan GS7470 Chipset Interface
     cantain all the information related to device 
     this interface is loaded depending on
     compile option (see ./configure --help | --with-chipset option)
*/

#include "gsinterface.h"
#include <stdlib.h>
#include <stdio.h>
#include <stdio.h>
#include <fcntl.h>
#include <dirent.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/time.h>

/* GSControlINT variable */
GSControlINT* gs;
/* GSControlReg pointer list for connections type (see CUR_CONN_#### defines) for code 0x7311*/
GSControlReg* connCtrlReg73[4];
/* GSControlReg pointer list for connections type (see CUR_CONN_#### defines) for code 0x5311*/
GSControlReg* connCtrlReg53[4];

/* Allocate a regster and setup some values */
GSControlReg* allocateGSCtrlReg(char cdType, unsigned int connType){
	
	GSControlReg* rp=(GSControlReg*)malloc(sizeof(GSControlReg));
	switch(connType){
		case CUR_CONN_F343 :
			switch(cdType){
				case 0x73 :
					rp->replace = malloc(2);
					rp->replace[0] = 0x63;
					rp->replace[1] = 0x01;
					rp->maxRepPosition=0;
					break;
				case 0x53 :
					rp->replace = malloc(2);
					rp->replace[0] = 0x43;
					rp->replace[1] = 0x01;
					rp->maxRepPosition=0;
					break;
				default: ;
			}
			break;
		case CUR_CONN_7341 :
			switch(cdType){
				case 0x73 :
					rp->replace = malloc(6);
					rp->replace[0] = 0x63;
					rp->replace[1] = 0x01;
					rp->replace[2] = 0xe3;
					rp->replace[3] = 0x47;
					rp->replace[4] = 0x63;
					rp->replace[5] = 0x51;
					rp->maxRepPosition=2;
					break;
				case 0x53 :
					rp->replace = malloc(8);
					rp->replace[0] = 0xc3;
					rp->replace[1] = 0xb5;
					rp->replace[2] = 0xc3;
					rp->replace[3] = 0x53;
					rp->replace[4] = 0x43;
					rp->replace[5] = 0x4f;
					rp->replace[6] = 0xc3;
					rp->replace[7] = 0x39;
					rp->maxRepPosition=3;					
					break;
				default: ;
			}
			break;
		case CUR_CONN_734D :
			switch(cdType){
				case 0x73 :
					rp->replace = malloc(2);
					rp->replace[0] = 0xe3;
					rp->replace[1] = 0x19;
					rp->maxRepPosition=0;
					break;
				case 0x53 :
					rp->replace = 0;
					rp->maxRepPosition=-1;
					break;
				default: ;
			}
			break;
		case CUR_CONN_F34F :
			switch(cdType){
				case 0x73 :
					rp->replace = malloc(2);
					rp->replace[0] = 0xe3; /* o 6347 o e345 */
					rp->replace[1] = 0x47;
					rp->maxRepPosition=0;
					break;
				case 0x53 :
					rp->replace = 0;
					rp->maxRepPosition=-1;
					break;
				default: ;
			}
			break;
		default : ;		
	}
	rp->count=0;
	rp->curRepPosition=0;
	return(rp);
}

/* Clean up a register */
void deallocateGSCtrlRegs(GSControlReg* gscrl[]){
	unsigned int i;
	for (i=0; i< sizeof(gscrl); i++){
		free(gscrl[i]);
	}
}

/* Allocate the gs7470 interrupt handler and do some initilisation */
void allocateGSint(void)
{
	/*which way round should a malloc be?*/
	gs=(GSControlINT*)malloc(sizeof(GSControlINT));

	/* Allocate GSCtrlReg for cod 0x7311 */
	connCtrlReg73[CUR_CONN_F343] = allocateGSCtrlReg(0x73, CUR_CONN_F343);
	connCtrlReg73[CUR_CONN_7341] = allocateGSCtrlReg(0x73, CUR_CONN_7341);	
	connCtrlReg73[CUR_CONN_734D] = allocateGSCtrlReg(0x73, CUR_CONN_734D);	
	connCtrlReg73[CUR_CONN_F34F] = allocateGSCtrlReg(0x73, CUR_CONN_F34F);

	/* Allocate GSCtrlReg for cod 0x5311 */
	connCtrlReg53[CUR_CONN_F343] = allocateGSCtrlReg(0x53, CUR_CONN_F343);
	connCtrlReg53[CUR_CONN_7341] = allocateGSCtrlReg(0x53, CUR_CONN_7341);	
	connCtrlReg53[CUR_CONN_734D] = allocateGSCtrlReg(0x53, CUR_CONN_734D);	
	connCtrlReg53[CUR_CONN_F34F] = allocateGSCtrlReg(0x53, CUR_CONN_F34F);

	gs->controlcodecount=-1;
	gs->controlseqcount=0;
	gs->curReplace73=0;
	gs->curReplace53=0;
	gs->currentConnection = CUR_CONN_UNKNOWN;
}

/* Deallocate the gs7470 interrupt handler and clean up */
void deallocateGSint(){
	gs->curReplace73=0;
	gs->curReplace53=0;
	free(gs);
	deallocateGSCtrlRegs(connCtrlReg73);
	deallocateGSCtrlRegs(connCtrlReg53);
}

/* Manages commands sent from the modem */

int gsSetControl(unsigned char* inBuffer)
{
	unsigned short controlcode=(inBuffer[0]<<8 )|inBuffer[1];
	/*  Check for junk/unknown stuff, because it probably means that the line
	    has dropped */
	if (controlcode==0x0C0C)
		return(0);
	/* check to see if we have a new control code to put into the buffer */
	switch(controlcode){
		case 0xf343 :
			gs->currentConnection = CUR_CONN_F343; 
			gs->curReplace73 = connCtrlReg73[CUR_CONN_F343];
			gs->curReplace73->count = 0;
			gs->curReplace73->curRepPosition = 0;
			gs->curReplace53 = connCtrlReg53[CUR_CONN_F343];
			gs->curReplace53->count = 0;
			gs->curReplace53->curRepPosition = 0;
			break;
		case 0x7341 :
			gs->currentConnection = CUR_CONN_7341;
			gs->curReplace73 = connCtrlReg73[CUR_CONN_7341];
			gs->curReplace73->count = 0;
			gs->curReplace73->curRepPosition = 0;
			gs->curReplace53 = connCtrlReg53[CUR_CONN_7341];
			gs->curReplace53->count = 0;
			gs->curReplace53->curRepPosition = 0;
			break;
		case 0x734d :
			gs->currentConnection = CUR_CONN_734D;
			gs->curReplace73 = connCtrlReg73[CUR_CONN_734D];
			gs->curReplace73->count = 0;
			gs->curReplace73->curRepPosition = 0;
			gs->curReplace53 = connCtrlReg53[CUR_CONN_734D];
			gs->curReplace53->count = 0;
			gs->curReplace53->curRepPosition = 0;
			break;
		case 0xf34f :
			gs->currentConnection = CUR_CONN_F34F;
			gs->curReplace73 = connCtrlReg73[CUR_CONN_F34F];
			gs->curReplace73->count = 0;
			gs->curReplace73->curRepPosition = 0;
			gs->curReplace53 = connCtrlReg53[CUR_CONN_F34F];
			gs->curReplace53->count = 0;
			gs->curReplace53->curRepPosition = 0;
			break;
		default: ;
	}
	return(0);
}

/* Returns the correct response to a control command sent by the modem,
   gsSetControl must be called first so that we are in the correct
   mode for the response.
*/
void gsGetResponse(unsigned char* buffer)
{
	if ((buffer[0]!=0x73 && buffer[0]!=0x53) || buffer[1]!=0x11)
		return; /* nothing to do */
	switch (buffer[0])
	{
		case 0x53:
			if (gs->curReplace53!=0){
				if (gs->curReplace53->count==0 && gs->curReplace73->maxRepPosition!=-1 && gs->curReplace73->count!=0){
					if (gs->curReplace73->curRepPosition < gs->curReplace73->maxRepPosition){
						gs->curReplace73->curRepPosition++;
					}else{
						gs->curReplace73->curRepPosition=0;
					}
					gs->curReplace73->count = 0;
				}
				/*BUG : check if curReplace53 has some replace codes */
				if (gs->curReplace53->maxRepPosition >0){
					gs->curReplace53->count++;
					if (gs->curReplace53->count<=2){
						/*don't need to replace anything the first two times around*/
						break;
					}
					buffer[0]= gs->curReplace53->replace[(gs->curReplace53->curRepPosition *2)];
					buffer[1]= gs->curReplace53->replace[(gs->curReplace53->curRepPosition *2)+1];
				}
			}
			break;
		case 0x73:
			if (gs->curReplace73!=0){
				if (gs->curReplace73->count==0 && gs->curReplace53->maxRepPosition!=-1 && gs->curReplace53->count!=0){
					if (gs->curReplace53->curRepPosition < gs->curReplace53->maxRepPosition){
						gs->curReplace53->curRepPosition++;
					}else{
						gs->curReplace53->curRepPosition=0;
					}
					gs->curReplace53->count = 0;
				}
				gs->curReplace73->count++;
				if (gs->curReplace73->count<=2){
					/*don't need to replace anything the first two times around*/
					break;
				}
				buffer[0]= gs->curReplace73->replace[(gs->curReplace73->curRepPosition *2)];
				buffer[1]= gs->curReplace73->replace[(gs->curReplace73->curRepPosition *2)+1];
			}
			break;
		default:;
	}
}


/* get structure for aal5 header */
/*aal5_header_st getAal5HeaderStructure(unsigned char* aal5Header){
		aal5_header_st hdst;

	hdst=(aal5_header_st)malloc(sizeof(aal5_header_st));
	hdst->vpi = ((aal5Header[0]  >>  4) & 0x0f);
	hdst->vci = (((aal5Header[1]  <<  4) & 0xf0) | ((aal5Header[2] >> 4) & 0x0f));
	hdst->pti = ( aal5Header[2] & 0x0f);			
	return(hdst);
}*/
